==> omnibuild-0.1.0%2B2.tar.gz: lib/src/parse.dart <==
// Copyright (c) 2015, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

part of omnibuild;

/// Parse YAMl data into an OmnibuildSpecFile instance.
void _parseYaml(OmnibuildSpecFile spec, String yamlString) {
  final yaml = loadYaml(yamlString);

  // Load copyright, library and description.
  spec.copyright = yaml.containsKey('copyright') ? yaml['copyright'] : '';
  spec.library = yaml.containsKey('library') ? yaml['library'] : '';
  spec.description = yaml.containsKey('description') ? yaml['description'] : '';

  // Loop through messages.
  if (yaml.containsKey('messages')) {
    yaml['messages'].forEach((YamlMap message) {
      spec.messages.add(_parseMessageYaml(message, spec.package, spec.version));
    });
  }
}

/// Parse YAML message data.
OmnibuildMessage _parseMessageYaml(YamlMap data, String package, int version) {
  // Message name (non-optional)
  final name = data['name'];

  // Get messate type as OmnibusMessageType.
  // Enforces lower-case convention on purpose.
  const messageTypes = const {
    'event': OmnibusMessageType.event,
    'request': OmnibusMessageType.request,
    'response': OmnibusMessageType.response
  };
  final type = messageTypes[(data.containsKey('type') &&
      messageTypes.containsKey(data['type']) ? data['type'] : 'event')];

  // Message documentation
  final documentation = data.containsKey('note') ? data['note'] : '';

  // Message properties
  var properties = new List<OmnibuildProperty>();
  if (data.containsKey('properties')) {
    data['properties'].forEach((YamlMap property) {
      properties.add(_parsePropertyYaml(property));
    });
  }

  return new OmnibuildMessage(
      '$package.v$version.$name', name, documentation, type, properties);
}

OmnibuildProperty _parsePropertyYaml(YamlMap data) {
  // name and type are non-optional
  final String name = data['name'];
  String type = data['type'];

  // Enumeration types
  bool enumeration = false;
  if (type.startsWith('enum ')) {
    enumeration = true;
    type = type.substring(5);
  }

  // Message documentation
  final String documentation = data.containsKey('note') ? data['note'] : '';

  // Pass property as named parameter in message builder.
  // Enforces lower-case convention on purpose.
  // Defaults to positional parameter (namedParameter = false).
  final bool namedParameter =
      data.containsKey('pass') ? data['pass'] == 'named parameter' : false;

  // Stringified default value. Only works for dart primitive types that can be
  // expressed as string in YAML (boolean, integer, list, map).
  final String defaultValue = _resolveDefaultValueCode(type,
      data.containsKey('dflt') ? data['dflt'].toString() : '', namedParameter);

  return new OmnibuildProperty(
      name, documentation, type, enumeration, namedParameter, defaultValue);
}

/// Generate default value for OmnibuildProperty.
String _resolveDefaultValueCode(String type, String dflt, bool namedParameter) {
  if (type == 'String') {
    if (dflt.isNotEmpty) {
      // Single line strings
      if (!dflt.contains('\n')) {
        if (!dflt.contains("'")) {
          // Use single quotes if the string contains no single quotes.
          return "'$dflt'";
        } else if (!dflt.contains('"')) {
          // Use double quotes if the string contains no double quotes.
          return '"$dflt"';
        } else {
          // Use single quotes and escape all single quotes in the string tha
          // are not already escaped (tricky).
          return "'${_escapeSingleQuotes(dflt)}'";
        }
      } else {
        if (!dflt.contains("'''")) {
          // Use triple quotes.
          return "'''$dflt'''";
        } else {
          // Escape all single quotes in the string and use triple quotes.
          return "'''${_escapeSingleQuotes(dflt)}'''";
        }
      }
    } else {
      return namedParameter ? "''" : '';
    }
  } else if (dflt.isEmpty) {
    // Generate standard dflt value if necessary.
    if (namedParameter) {
      if (type.startsWith('List<')) {
        return 'const []';
      } else if (type.startsWith('Map<')) {
        return 'const {}';
      } else {
        return 'null';
      }
    } else {
      return '';
    }
  } else {
    return dflt;
  }
}

String _escapeSingleQuotes(String src) {
  return src.replaceAll("'", "\\'");
}

==> omnibuild-0.5.2%2B2.tar.gz: lib/src/parse.dart <==
// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by a MIT-style license
// that can be found in the LICENSE file.

part of omnibuild;

/// Parse YAMl data into an OmnibuildSpecFile instance.
void _parseYaml(OmnibuildSpecFile spec, String yamlString) {
  final yaml = loadYaml(yamlString);

  // Load copyright, library and description.
  spec.copyright = yaml.containsKey('copyright') ? yaml['copyright'] : '';
  spec.library = yaml.containsKey('library') ? yaml['library'] : '';
  spec.description = yaml.containsKey('description') ? yaml['description'] : '';

  // Loop through messages.
  if (yaml.containsKey('messages')) {
    yaml['messages'].forEach((YamlMap message) {
      spec.messages.add(_parseMessageYaml(message, spec.package, spec.version));
    });
  }
}

/// Parse YAML message data.
OmnibuildMessage _parseMessageYaml(YamlMap data, String package, int version) {
  final String name = data['name'];

  // Get messate type as OmnibusMessageType.
  // Enforces lower-case convention on purpose.
  const messageTypes = const {
    'event': OmnibusMessageType.event,
    'request': OmnibusMessageType.request,
    'response': OmnibusMessageType.response
  };
  final type = messageTypes[(data.containsKey('type') &&
      messageTypes.containsKey(data['type']) ? data['type'] : 'event')];

  // Message properties
  var properties = new List<OmnibuildProperty>();
  var filter = new List<OmnibuildProperty>();
  if (data.containsKey('properties')) {
    data['properties'].forEach((YamlMap property) {
      properties.add(_parsePropertyYaml(property));

      // If the property YAML map contains a 'fltr': true field, the property
      // index is added to the filter fields.
      if (property.containsKey('fltr') && property['fltr'] == true) {
        filter.add(properties.last);
      }
    });
  }

  return new OmnibuildMessage('$package.v$version.$name', name,
      data.containsKey('note') ? data['note'] : '', type, properties, filter);
}

OmnibuildProperty _parsePropertyYaml(YamlMap data) {
  // name and type are non-optional
  String type = data['type'];

  // Enumeration types
  bool enumeration = false;
  if (type.startsWith('enum ')) {
    enumeration = true;
    type = type.substring(5);
  }

  // A PDS type should be defined when the type contains classes that are non-
  // standard.
  String pdsType = data.containsKey('data') ? data['data'] : '';

  // If no PDS type is given, and the type contains Sets and/or int Maps,
  // convert the type to a PDS type by replacing Set< with List< and Map<int,
  // with Map<String,.
  if (pdsType.isEmpty && (type.contains('Set<') || type.contains('Map<'))) {
    pdsType =
        type.replaceAll('Set<', 'List<').replaceAll('Map<int,', 'Map<String,');
  }

  if (pdsType.isNotEmpty) {
    // Check if the PDS type and the type are in conflict.
    // Note: if this is an enumeration the pdsType must be set to empty or int.
    if ((enumeration && pdsType != 'int') ||
        !isComplementaryPdsType(type, pdsType)) {
      throw new ArgumentError.value(
          data, 'data', 'a PDS type should be complementary to the full type');
    }
  } else if (!enumeration && !isPdsType(type)) {
    throw new ArgumentError.value(data, 'data',
        'a PDS type must be defined when the full type is not already a PDS');
  }

  // Pass property as named parameter in message builder.
  // Enforces lower-case convention on purpose.
  // Defaults to positional parameter (namedParameter = false).
  final bool namedParameter =
      data.containsKey('pass') ? data['pass'] == 'named parameter' : false;

  // Stringified default value. Only works for dart primitive types that can be
  // expressed as string in YAML (boolean, integer, list, map).
  final String defaultValue = _resolveDefaultValueCode(type,
      data.containsKey('dflt') ? data['dflt'].toString() : '', namedParameter);

  return new OmnibuildProperty(
      data['name'],
      data.containsKey('note') ? data['note'] : '',
      type,
      pdsType,
      enumeration,
      namedParameter,
      defaultValue);
}

/// Generate default value for OmnibuildProperty.
String _resolveDefaultValueCode(String type, String dflt, bool namedParameter) {
  if (type == 'String') {
    if (dflt.isNotEmpty) {
      // Single line strings
      if (!dflt.contains('\n')) {
        if (!dflt.contains("'")) {
          // Use single quotes if the string contains no single quotes.
          return "'$dflt'";
        } else if (!dflt.contains('"')) {
          // Use double quotes if the string contains no double quotes.
          return '"$dflt"';
        } else {
          // Use single quotes and escape all single quotes in the string tha
          // are not already escaped (tricky).
          return "'${_escapeSingleQuotes(dflt)}'";
        }
      } else {
        if (!dflt.contains("'''")) {
          // Use triple quotes.
          return "'''$dflt'''";
        } else {
          // Escape all single quotes in the string and use triple quotes.
          return "'''${_escapeSingleQuotes(dflt)}'''";
        }
      }
    } else {
      return namedParameter ? "''" : '';
    }
  } else if (dflt.isEmpty) {
    // Generate standard dflt value if necessary.
    if (namedParameter) {
      if (type.startsWith('List<')) {
        return 'const []';
      } else if (type.startsWith('Map<')) {
        return 'const {}';
      } else {
        return 'null';
      }
    } else {
      return '';
    }
  } else {
    return dflt;
  }
}

String _escapeSingleQuotes(String src) {
  return src.replaceAll("'", "\\'");
}
